// -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-

/*
 * Copyright (C) 2006 Paul Fitzpatrick
 * CopyPolicy: Released under the terms of the LGPLv2.1 or later, see LGPL.TXT
 *
 */


#include <yarp/os/impl/Logger.h>

//#include <ace/OS.h>
#include <ace/OS_NS_stdlib.h>
#include <ace/OS_NS_stdio.h>
#include <ace/OS_NS_unistd.h>
#include <ace/Thread.h>

using namespace yarp::os::impl;

Logger Logger::root("yarp");

Logger& Logger::get() {
    return root;
}

#ifdef __linux__
#include <execinfo.h>
void yarp_print_trace(FILE *out, const char *file, int line) {
    const size_t max_depth = 100;
    size_t stack_depth;
    void *stack_addrs[max_depth];
    char **stack_strings;
    stack_depth = backtrace(stack_addrs, max_depth);
    stack_strings = backtrace_symbols(stack_addrs, stack_depth);
    fprintf(out, "Assertion thrown at %s:%d by code called from:\n", file, line);
    for (size_t i = 1; i < stack_depth; i++) {
        fprintf(out, " --> %s\n", stack_strings[i]);
    }
    free(stack_strings); // malloc()ed by backtrace_symbols
    fflush(out);
}
#else
void yarp_print_trace(FILE *out, const char *file, int line) {
    // Not implemented on this platform
}
#endif

void Logger::show(int level, const String& txt) {
    int inLevel = level;
    //ACE_OS::fprintf(stderr,"level %d txt %s\n", level, txt.c_str());
    if (verbose>0) {
        level = 10000;
    }
    if (verbose<0) {
        level = -10000;
    }
    if (parent == NULL) {
        if (level>=low) {
            if (inLevel<=DEBUG) {
                ACE_OS::fprintf(stderr,"%s(%04x): %s\n",
                                prefix.c_str(),
                                (long int)(ACE_Thread::self()),
                                txt.c_str());
            } else {
                ACE_OS::fprintf(stderr,"%s: %s\n",prefix.c_str(),txt.c_str());
            }
            ACE_OS::fflush(stderr);
        }
    } else {
        String more(prefix);
        more += ": ";
        more += txt;
        parent->show(inLevel,more);
    }
}


void Logger::exit(int level) {
    ACE_OS::exit(level);
}


void Logger::setPid() {
    pid = ACE_OS::getpid();
}


void __yarp_error(const char *str) {
    YARP_ERROR(Logger::get(),str);
}

void __yarp_warn(const char *str) {
    YARP_WARN(Logger::get(),str);
}

void __yarp_info(const char *str) {
    YARP_INFO(Logger::get(),str);
}

void __yarp_debug(const char *str) {
    YARP_DEBUG(Logger::get(),str);
}
